/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Australis panel script.
 *
 * ************************************************************************
 */

Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import("resource://gre/modules/devtools/Console.jsm");
Components.utils.import("resource:///modules/CustomizableUI.jsm");
Components.utils.import("resource://cookiekeeper/cookiekeeper.jsm");

var currentBehavior = 0;
var lifetimePolicy  = 0;
var l10n            = null;

function hidePanel() {
  CustomizableUI.hidePanelForNode(window.document.ckPanel);
  window.document.ckPanel = null;
}

function showPref(event) {
  CookieKeeper.showPreferencesDialogue(window, null);
  hidePanel();
  event.stopPropagation();
}

function showMainWin(event) {
  CookieKeeper.showMainWindow(window);
  hidePanel();
  event.stopPropagation();
}

function showCookiesOf(event, domain) {
  CookieKeeper.showMainWindow(window, domain);
  hidePanel();
  event.stopPropagation();
}

function clearCookies(event) {
  hidePanel();
  CookieKeeper.clearAllCookies(false);
  event.stopPropagation();
}

function showPerm(perm, number, _default) {
  const SHIFT = 24;
  let container = document.getElementById("cookiekeeper-panel-perms-div-selector-" + number);
  let selector = document.getElementById("cookiekeeper-panel-perms-img-selector-" + number);
  if (typeof(_default) != "boolean") {
    _default = false;
  }

  switch (perm) {
  case MOZ_COOKIE_PERM_ALLOW:
    if (_default) {
      selector.style.marginLeft = "0px";
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-check-off-24.png");
    } else {
      selector.style.marginLeft = (SHIFT).toString(10) + "px";
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-green-check-off-24.png");
    }
    container.setAttribute("tooltiptext", l10n.tr("tooltiptext.allow"));
    break;
  case MOZ_COOKIE_PERM_SESSION:
    if (_default) {
      selector.style.marginLeft = "0px";
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-barre-off-24.png");
    } else {
      selector.style.marginLeft = (SHIFT*2).toString(10) + "px";
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-yellow-barre-off-24.png");
    }
    container.setAttribute("tooltiptext", l10n.tr("tooltiptext.session"));
    break;
  case MOZ_COOKIE_PERM_DENY:
    if (_default) {
      selector.style.marginLeft = "0px";
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-cross-off-24.png");
    } else {
      selector.style.marginLeft = (SHIFT*3).toString(10) + "px";
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-red-cross-off-24.png");
    }
    container.setAttribute("tooltiptext", l10n.tr("tooltiptext.block"));
    break;
  case MOZ_COOKIE_PERM_DEFAULT:
  default:
    selector.style.marginLeft = "0px";
    if (currentBehavior == MOZ_BEHAVIOR_DISABLED) {
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-cross-off-24.png");
      container.setAttribute("tooltiptext", l10n.tr("tooltiptext.block"));
    } else if (lifetimePolicy == MOZ_ACCEPT_SESSION) {
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-barre-off-24.png");
      container.setAttribute("tooltiptext", l10n.tr("tooltiptext.session"));
    } else if (lifetimePolicy == MOZ_ASK_BEFORE_ACCEPT) {
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-question-off-24.png");
      container.setAttribute("tooltiptext", l10n.tr("tooltiptext.ask"));
    } else if (lifetimePolicy == MOZ_ACCEPT_FOR_N_DAYS) {
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-n_days-off-24.png");
      container.setAttribute("tooltiptext", l10n.tr("tooltiptext.limit"));
    } else {
      selector.setAttribute("src", "chrome://cookiekeeper/skin/indicator-grey-check-off-24.png");
      container.setAttribute("tooltiptext", l10n.tr("tooltiptext.allow"));
    }
    break;
  }
}

function onChangePerms(event) {
  let container = event.target;
  let number = container.getAttribute("number");
  if (container.localName == "img") {
    container = document.getElementById("cookiekeeper-panel-perms-div-selector-" + number);
  }
  let rect = container.getClientRects()[0];
  let domain = null;
  if (container.hasAttribute("domain")) {
    domain = container.getAttribute("domain");
  } else {
    log.e("Error: domain not set", event);
    return;
  }

  let clickPos = Math.floor(4*(event.clientX - rect.x) / (rect.width));
  clickPos = clickPos < 0 ? 0 : clickPos;
  clickPos = clickPos > 3 ? 3 : clickPos;

  var uri = Services.io.newURI("http://" + domain, null, null);
  let newPerm = MOZ_COOKIE_PERM_DEFAULT;

  switch (clickPos) {
  case 0:
    try {
      Services.perms.remove(domain, "cookie");
    } catch (ex) {
      Services.perms.remove(uri, "cookie");
    }
    break;
  case 1:
    newPerm = MOZ_COOKIE_PERM_ALLOW;
    Services.perms.add(uri, "cookie", MOZ_COOKIE_PERM_ALLOW);
    break;
  case 2:
    newPerm = MOZ_COOKIE_PERM_SESSION;
    Services.perms.add(uri, "cookie", MOZ_COOKIE_PERM_SESSION);
    break;
  case 3:
    newPerm = MOZ_COOKIE_PERM_DENY;
    Services.perms.add(uri, "cookie", MOZ_COOKIE_PERM_DENY);
    break;
  }
  showPerm(newPerm, number, false);

  // Refresh sub-domain permissions:
  let inumber = parseInt(number) - 1;
  number    = inumber.toString(10);
  container = document.getElementById("cookiekeeper-panel-perms-div-selector-" + number);
  while (inumber >= 0 && container != null) {
    domain = null;
    if (container.hasAttribute("domain")) {
      domain = container.getAttribute("domain");

      uri = Services.io.newURI("http://" + domain, null, null);
      let defaultPerm = Services.perms.testPermission(uri, "cookie");
      let exactPerm   = Services.perms.testExactPermission(uri, "cookie");
      let currentPerm = defaultPerm == MOZ_COOKIE_PERM_DEFAULT ? exactPerm : defaultPerm;
      showPerm(currentPerm, number, exactPerm == MOZ_COOKIE_PERM_DEFAULT);

      inumber--;
      number    = inumber.toString(10);
      container = document.getElementById("cookiekeeper-panel-perms-div-selector-" + number);
    } else {
      container = null;
    }
  }

  event.stopPropagation();
}

function translateUI(body) {
  function translateDomItem(domItem) {
    let nChildren = domItem.children.length;
    for (let i=0; i<nChildren; i++) {
      let child = domItem.children[i];
      if (child.hasAttribute("data-l10n-id")) {
        child.textContent = l10n.tr(child.getAttribute("data-l10n-id"));
      }
      if (child.hasAttribute("data-l10n-id-tooltiptext")) {
        child.setAttribute("tooltiptext", l10n.tr(child.getAttribute("data-l10n-id-tooltiptext")));
      }
      translateDomItem(child);
    }
  }
  translateDomItem(body);
}

function main(aEvent) {
  let panel  = window.document.ckPanel;
  let domain = window.document.ckDomain;

  l10n = window.document.l10n;
  translateUI(document.getElementById("cookiekeeper-panel-body"));

  if (domain && domain.length > 0) {
    let minDomain = "";
    let domains = [];
    try {
      minDomain = Services.eTLD.getBaseDomainFromHost(domain);
      domains   = domain.split(".");
    } catch (e) {
      // probably an IP address
      domains = [domain];
      minDomain = domain;
    }
    let divPerms  = document.getElementById("cookiekeeper-panel-perms-list");

    let appPrefInfos = APP_MOZ_CONST[Services.appinfo.ID];
    if (Services.prefs.getPrefType(appPrefInfos.COOKIES_BEHAVIOR) == Services.prefs.PREF_INT) {
      currentBehavior = Services.prefs.getIntPref(appPrefInfos.COOKIES_BEHAVIOR);
    }
    if (Services.prefs.getPrefType(appPrefInfos.COOKIES_LIFETIME) == Services.prefs.PREF_INT) {
      lifetimePolicy = Services.prefs.getIntPref(appPrefInfos.COOKIES_LIFETIME);
    }

    for (let i=0; i<domains.length; i++) {
      let partialDomain = domains[i];
      for (let j=i+1; j<domains.length; j++) {
        partialDomain = partialDomain + "." + domains[j];
      }
      if (minDomain.length > partialDomain.length) {
        break;
      }
      let uri = Services.io.newURI("http://" + partialDomain, null, null);
      let defaultPerm = Services.perms.testPermission(uri, "cookie");
      let exactPerm   = Services.perms.testExactPermission(uri, "cookie");
      let currentPerm = defaultPerm == MOZ_COOKIE_PERM_DEFAULT ? exactPerm : defaultPerm;

      let number      = i.toString(10);
      let divHoriz    = document.createElement("div");
      let spanDomain  = document.createElement("span");
      let divSelector = document.createElement("div");
      let imgSelector = document.createElement("img");

      divHoriz.setAttribute("class", "cookiekeeper-panel-div-table-row");
      spanDomain.setAttribute("class", "cookiekeeper-panel-span-domain");
      spanDomain.setAttribute("onclick", "showCookiesOf(event, '" + partialDomain + "');");
      spanDomain.textContent = partialDomain;
      divHoriz.appendChild(spanDomain);
      divSelector.setAttribute("id", "cookiekeeper-panel-perms-div-selector-" + number);
      divSelector.setAttribute("class", "cookiekeeper-panel-perms-div-selector");
      divSelector.setAttribute("onclick", "onChangePerms(event);");
      divSelector.setAttribute("number", number);
      divSelector.setAttribute("domain", partialDomain);
      imgSelector.setAttribute("id", "cookiekeeper-panel-perms-img-selector-" + number);
      imgSelector.setAttribute("class", "cookiekeeper-panel-perms-img-selector");
      imgSelector.setAttribute("number", number);
      imgSelector.setAttribute("domain", partialDomain);
      divSelector.appendChild(imgSelector);
      divHoriz.appendChild(divSelector);
      divPerms.appendChild(divHoriz);

      showPerm(currentPerm, number, exactPerm == MOZ_COOKIE_PERM_DEFAULT);
      window.setTimeout(function() {imgSelector.style.transitionDuration = "0.1s";}, 100);
    }
  } else {
    // The current tab seems to not be on a website, probably an about:*
    // remove it.
    let divPerm = document.getElementById("cookiekeeper-panel-perms");
    divPerm.parentNode.removeChild(divPerm);
  }

  // adjust the panel size
  if (window.scrollMaxX > 0) {
    panel.style.width = (window.innerWidth + window.scrollMaxX).toString(10) + "px";
    panel.firstChild.style.width = (window.innerWidth + window.scrollMaxX).toString(10) + "px";
  }
  if (window.scrollMaxY > 0) {
    panel.style.height = (window.innerHeight + window.scrollMaxY).toString(10) + "px";
    panel.firstChild.style.height = (window.innerHeight + window.scrollMaxY).toString(10) + "px";
  }

  // re-adjust the panel arrow position
  let popup = panel.parentElement;
  while (popup.tagName != "panel") {
    popup = popup.parentElement;
  }
  popup.adjustArrowPosition();
}
